#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <signal.h>
#include <limits.h>
#include <sys/prctl.h>
#include <sys/schedctl.h>
#include "trackd.h"

#define PERMS 0666

static int controller_shmid=-1;
static struct TRACKD_CONTROLLER	*controller=NULL;


void init_controller(void)
{
 /**** Insert your code here ****/
}


int get_new_controller_data(CAVE_CONTROLLER_ST *data)
{
 /**** Insert your code here
    Return 1 if new data was read, 0 if not ****/
}


/*************************************************************************
 void trackd_cleanup()
	When the daemon is interrupted by SIGINT, this routine closes the
 connections, disposes of the memory, and exits.
**************************************************************************/
void trackd_cleanup(void)
{
 fprintf(stderr,"Interrupted.  Quitting...\n");
 if (controller)
	if (shmdt(controller) < 0)
		perror("Detaching 'controller' shared memory");
 if (controller_shmid > -1)
	if (shmctl(controller_shmid,IPC_RMID) < 0)
		perror("Removing 'controller_shmid'");
 exit(-1);
}


/*************************************************************************
 void get_controller_mem()
	Allocates shared memory for the buttons/joystick.  The global
  'controller' points to the chunk.
**************************************************************************/
void get_controller_mem(CAVE_CONFIG_ST *config)
{
 controller_shmid = shmget(config->ControllerdKey, 
			sizeof(struct TRACKD_CONTROLLER), PERMS | IPC_CREAT);
 if (controller_shmid < 0)
	{
	fprintf(stderr,"can't get shared memory\n");
	exit(-1);
	}
 controller = (struct TRACKD_CONTROLLER *) 
			shmat(controller_shmid,(char *) 0, 0);
 if (controller == (struct TRACKD_CONTROLLER *) -1)
	{
	fprintf(stderr,"can't attach shared memory\n");
	exit(-1);
	}
}


/*************************************************************************
 void init_controller_data()
	Fills in the controller header and data with initial values.
**************************************************************************/
void init_controller_data(CAVE_CONFIG_ST *config)
{
 int loop;

 controller->header.version = CAVELIB_2_6;
 controller->header.buttonOffset = ((char *)&controller->controller.button[0]) -
					((char *)controller);
 controller->header.valuatorOffset = ((char *)&controller->controller.valuator[0]) -
					((char *)controller);
 controller->header.numButtons = 3;
 controller->header.numValuators = 2;
 controller->header.timestamp[0] = controller->header.timestamp[1] = 0;
 controller->header.command = 0;

 for (loop=0; loop < CAVE_MAX_BUTTONS; ++loop) 
		controller->controller.button[loop] = 0;
 for (loop=0; loop < CAVE_MAX_VALUATORS; ++loop) 
		controller->controller.valuator[loop] = 0.0;
 controller->controller.num_buttons = 3;
 controller->controller.num_valuators = 2;
}


void get_timestamp(uint32_t stamp[2])
{
 struct timeval curtime;
 gettimeofday(&curtime,NULL);
 stamp[0] = curtime.tv_sec;
 stamp[1] = curtime.tv_usec;
}


void process_controller_command(int command)
{
}


main(int argc,char **argv)
{
 int loop;
 CAVE_CONFIG_ST config;
 CAVE_CONTROLLER_ST controlvals;

 signal(SIGINT,trackd_cleanup); /* catch interrupts to close nicely */
 schedctl(NDPRI,0,NDPHIMIN);  /* set a non-degrading priority */

 if (!getenv("CAVEDEBUGCONFIG"))
        putenv("CAVEDEBUGCONFIG=OFF");
 CAVEConfigurationInit(&argc,argv,NULL,&config);
	
 get_controller_mem(&config);

 init_controller_data(&config);

 init_controller();

 while (1)
	{
	if (get_new_controller_data(&controlvals))
		{
		controller->controller = controlvals;
		get_timestamp(controller->header.timestamp);
		}
	if (controller->header.command)
		{
		process_controller_command(controller->header.command);
		controller->header.command = 0;
		}
	}
}
